@@ -1,4 +1,4 @@
-# encoding: utf-8
+# frozen_string_literal: true
 
 require "json"
 require "money/currency/loader"
@@ -9,7 +9,7 @@ class Money
   # Represents a specific currency unit.
   #
   # @see https://en.wikipedia.org/wiki/Currency
-  # @see http://iso4217.net/
+  # @see https://www.iso.org/iso-4217-currency-codes.html
   class Currency
     include Comparable
     extend Enumerable
@@ -33,6 +33,9 @@ class Money
     # Thrown when an unknown currency is requested.
     class UnknownCurrency < ArgumentError; end
 
+    # Thrown when currency is not provided.
+    class NoCurrency < ArgumentError; end
+
     class << self
       def new(id)
         id = id.to_s.downcase
@@ -80,8 +83,8 @@ class Money
       def find_by_iso_numeric(num)
         num = num.to_s.rjust(3, '0')
         return if num.empty?
-        id, _ = self.table.find { |key, currency| currency[:iso_numeric] == num }
-        new(id)
+        id = iso_numeric_index[num]
+        new(id) if id
       rescue UnknownCurrency
         nil
       end
@@ -113,13 +116,13 @@ class Money
       #
       # == monetary unit
       # The standard unit of value of a currency, as the dollar in the United States or the peso in Mexico.
-      # https://www.answers.com/topic/monetary-unit
+      # https://www.answers.com/redirectSearch?query=monetary-unit
       # == fractional monetary unit, subunit
       # A monetary unit that is valued at a fraction (usually one hundredth) of the basic monetary unit
-      # https://www.answers.com/topic/fractional-monetary-unit-subunit
+      # https://www.answers.com/redirectSearch?query=fractional-monetary-unit-subunit
       #
       # See https://en.wikipedia.org/wiki/List_of_circulating_currencies and
-      # http://search.cpan.org/~tnguyen/Locale-Currency-Format-1.28/Format.pm
+      # https://metacpan.org/release/TNGUYEN/Locale-Currency-Format-1.28/view/Format.pm
       def table
         @table ||= Loader.load_currencies
       end
@@ -142,7 +145,7 @@ class Money
 
       # We need a string-based validator before creating an unbounded number of
       # symbols.
-      # http://www.randomhacks.net/articles/2007/01/20/13-ways-of-looking-at-a-ruby-symbol#11
+      # http://www.randomhacks.net.s3-website-us-east-1.amazonaws.com/2007/01/20/13-ways-of-looking-at-a-ruby-symbol/#11
       # https://github.com/RubyMoney/money/issues/132
       #
       # @return [Set]
@@ -170,8 +173,9 @@ class Money
       def register(curr)
         key = curr.fetch(:iso_code).downcase.to_sym
         @@mutex.synchronize { _instances.delete(key.to_s) }
-        @table[key] = curr
+        table[key] = curr
         @stringified_keys = nil
+        clear_iso_numeric_cache
       end
 
       # Inherit a new currency from existing one
@@ -180,7 +184,7 @@ class Money
       # @param curr [Hash] See {register} method for hash structure
       def inherit(parent_iso_code, curr)
         parent_iso_code = parent_iso_code.downcase.to_sym
-        curr = @table.fetch(parent_iso_code, {}).merge(curr)
+        curr = table.fetch(parent_iso_code, {}).merge(curr)
         register(curr)
       end
 
@@ -209,10 +213,21 @@ class Money
       def reset!
         @@instances = {}
         @table = Loader.load_currencies
+        clear_iso_numeric_cache
       end
 
       private
 
+      def iso_numeric_index
+        @iso_numeric_index ||= table.each_with_object({}) do |(id, attrs), index|
+          index[attrs[:iso_numeric]] = id
+        end
+      end
+
+      def clear_iso_numeric_cache
+        @iso_numeric_index = nil
+      end
+
       def stringify_keys
         table.keys.each_with_object(Set.new) { |k, set| set.add(k.to_s.downcase) }
       end
@@ -405,7 +420,7 @@ class Money
       !!@symbol_first
     end
 
-    # Returns if a code currency is ISO.
+    # Returns true if a code currency is ISO.
     #
     # @return [Boolean]
     #
@@ -416,6 +431,17 @@ class Money
       iso_numeric && iso_numeric != ''
     end
 
+    # Returns true if a subunit is cents-based.
+    #
+    # @return [Boolean]
+    #
+    # @example
+    #  Money::Currency.new(:usd).cents_based?
+    #
+    def cents_based?
+      subunit_to_unit == 100
+    end
+
     # Returns the relation between subunit and unit as a base 10 exponent.
     #
     # Note that MGA and MRU are exceptions and are rounded to 1
